;*_____________________________________________________________________________
;*
;*           Author: Terry Carmen 
;* Written by Terry Carmen 1/1/1992, Released to the public domain
;*_____________________________________________________________________________
;*
;*
;*     Program Name:   resolve.asm 
;*        $Revision$
;*            $Date$
;*
;*     Purpose: Resolves a relative path to a full path including drive 
;*
;*     Large model ASM function to be called from C
;*     
;*     char * resolve_path(char *)
;*
;*     called with a pointer to null terminated string containing 
;*     an absolute or relative path
;*     
;*     returns a pointer to a null terminated string, containing a
;*     fully qualified path, including drive and trailing backslash
;*     
;*     returns pointer to empty string if path is impossible to resolve
;*     
;*     note that it is possible for it to return a path that doesn't exist
;*     
;*     compile with TASM (or MASM) /mx /w2 
;*     
;*_____________________________________________________________________________


dosseg
model large

public  _resolve_path

.data

note             db      'resolve_path was written by Terry Carmen, '
                 db      '1/1/1992 and is released to the public domain'

path_buffer      db      16 dup ('pathbuff')      ; 128 byte buffer
BUF_LEN          equ     $-path_buffer



.code

_resolve_path   proc    far

                ;****************************************
                ;*  save the registers
                ;****************************************

                push    bp            ; we always need to preserve these
                mov     bp,sp       
                push    ds
                push    es
                push    si
                push    di


                ;****************************************
                ;*  point DS:SI to the input string
                ;****************************************

                mov ax, [bp+8]     ; segment for input string 
                mov ds, ax

                mov si, [bp+6]     ; offset for input string 



                ;****************************************
                ;*  point ES:DI to our buffer
                ;****************************************

                mov ax, @data
                mov es, ax

                mov di, offset path_buffer 



                ;****************************************
                ;* let DOS resolve the path
                ;****************************************


                mov ah, 60h        
                int 21h
                jc @@bye           ; if it can't, we're all done

                ; if the carry flag is set, it means that DOS couldn't 
                ; resolve the path correctly. therefore, we jump to @@bye. 


                ;****************************************
                ;* DOS found the path. 
                ;* all we have to do it put a \ and a null at the end
                ;****************************************

                cld
                mov ax, 0           ; look for a null
                mov cx, BUF_LEN     ; stop scanning when we run out of buffer
                repne scasb         ; or when AX=ES:[DI]

                cmp cx, 0           ; did we run out of string?
                jne @@search_ok     ; nope. we had a hit
                
                ; at this point, the search failed because 
                ; DOS gave us a longer string than we had buffer,
                ; or didn't terminate it with a null

                ; in either case, we're done, 
                ; and we'll just return a 0 length string


                mov di, offset path_buffer          
                ; point to the beginning of our buffer

                jmp short @@bye

@@search_ok:

                cmp byte ptr es:[di-2], '\' ; if it has a \, don't add one
                                            ; sometimes DOS returns it with 
                                            ; one, sometimes not . . .
                jne @@add_slash
                jmp short @@bye

@@add_slash:    mov byte ptr es:[di-1], '\' ; tack on a trailing backslash

@@bye:          mov byte ptr es:[di], 0     ; and fix the null terminator


                mov ax, offset path_buffer  ; string pointers are passed back 
                mov dx, ds                  ; in DX:AX

                pop     di
                pop     si
                pop     es
                pop     ds
                pop     bp
                ret   
                
_resolve_path   endp
                end








